#****************************************************************************
#
#  Module: run.ps1 for secRMM Az:Group:?
#
#  Purpose: Azure Function for secRMM Property Variables in Azure.
#
#  Copyright (c) 2023 Squadra Technologies 
#   
#****************************************************************************

# Group.Read.All
# GroupMember.Read.All
# User.Read.All

using namespace System.Net

# Input bindings are passed in via param block.
param($Request, $TriggerMetadata)

#=============================================================================
function Convert-AzureAdObjectIdToSid {
	param([String] $ObjectId)
	$bytes = [Guid]::Parse($ObjectId).ToByteArray()
	$array = New-Object 'UInt32[]' 4
	[Buffer]::BlockCopy($bytes, 0, $array, 0, 16)
	$sid = "S-1-12-1-$array".Replace(' ', '-')
	Write-Host "Convert-AzureAdObjectIdToSid: sid: $sid";
	return $sid
}

#=============================================================================
# Write to the Azure Functions log stream.
Write-Host "PowerShell HTTP trigger function is processing a secRMM request for Group members."

# Interact with query parameters or the body of the request.
$AzGroupName = $Request.Query.AzGroupName;

Write-Host "AzGroupName: $AzGroupName";

$body = "";

if ($AzGroupName) {
	$msiEndpoint = $env:MSI_ENDPOINT;
	$msiSecret = $env:MSI_SECRET;

	Write-Host "msiEndpoint: $msiEndpoint";
	# Write-Host "msiSecret: $msiSecret";

	$apiVersion = "2017-09-01";
	$resourceURI = "https://graph.microsoft.com";
	$tokenAuthURI = $msiEndpoint + "?resource=$resourceURI&api-version=$apiVersion";
	$tokenResponse = Invoke-RestMethod -Method Get -Headers @{"Secret"="$msiSecret"} -Uri $tokenAuthURI;
	$accessToken = $tokenResponse.access_token;

	$graphApiVersion = "v1.0";
	$graphApiHeader = @{
            'Content-Type'='application/json'
            'Authorization'="Bearer " + $accessToken
		    'ConsistencyLevel'='eventual'
            };
	
	$graphURI = 'https://graph.microsoft.com/' + $graphApiVersion + '/groups?$filter=displayName eq ' + '''' + $AzGroupName + '''' + ' and securityEnabled eq true' + '&$select=id';
	Write-Host "graphURI(1): $graphURI";

	$groupId = (Invoke-RestMethod -Uri $graphURI -Headers $graphApiHeader -Method Get).Value.id;
	Write-Host "graphURI(1) returned from call...";

	if ($groupId) {
		Write-Host "groupId: $groupId";

		$getUserPrincipalNames = $false;
		if ($getUserPrincipalNames -eq $true) {
			$graphURI = 'https://graph.microsoft.com/' + $graphApiVersion + '/groups/' + $groupId + '/members?$select=userPrincipalName';
			$userPrincipalNames = (Invoke-RestMethod -Uri $graphURI -Headers $graphApiHeader -Method Get).Value;

			if ($userPrincipalNames) {
				$body = ($userPrincipalNames | select -expand userPrincipalName) -join ";";
			}
		}
		else {
			$graphURI = 'https://graph.microsoft.com/' + $graphApiVersion + '/groups/' + $groupId + '/members?$select=id, onPremisesSecurityIdentifier';
			Write-Host "graphURI(2): $graphURI";
			$graphResponse = (Invoke-RestMethod -Uri $graphURI -Headers $graphApiHeader -Method Get);
			$userObjectIds = $graphResponse.Value;
			Write-Host "graphURI(2) returned from call...";
			$userCount = 1;
			$userIdentifiers = [System.Collections.ArrayList]@();
			foreach ($userObjectId in $userObjectIds) {
				Write-Host "userObjectId.id: $($userObjectId.id)";
				$SID1 = Convert-AzureAdObjectIdToSid -ObjectId $userObjectId.id;
				$SID2 = "";
				if ($userObjectId.onPremisesSecurityIdentifier -ne $null) {
					$SID2 = $userObjectId.onPremisesSecurityIdentifier;
				}		
				$SID = "{0}/{1}" -f $SID1, $SID2;
				Write-Host "userCount: $userCount ObjectId: $($userObjectId.id) ==> SIDs: $SID";
				$userIdentifiers.Add($SID) | Out-Null;
				$userCount += 1;
			}

			# https://learn.microsoft.com/en-us/graph/paging?tabs=http
			$ErrorActionPreference = "Stop";
			$nextLink = $null;
			Try {
				$nextLink = $graphResponse.'@odata.nextLink';
				Write-Host "nextLink: $nextLink";
			}
			Catch {
				Write-Host "error (or end of stream)[1] getting odata.nextLink: $_";
			}
			Finally {
				$ErrorActionPreference = "Continue"
			}

			$nextLinkCount = 0;
			while ($nextLink -ne $null) {
				$nextLinkCount += 1;
				Write-Host "graphURI(3): $nextLink";
				$graphResponse2 = (Invoke-RestMethod -Uri $nextLink -Headers $graphApiHeader -Method Get);
				$userObjectIds2 = $graphResponse2.Value;
				
				foreach ($userObjectId in $userObjectIds2) {
					Write-Host "userObjectId.id: $($userObjectId.id)";
					$SID1 = Convert-AzureAdObjectIdToSid -ObjectId $userObjectId.id;
					$SID2 = "";
					if ($userObjectId.onPremisesSecurityIdentifier -ne $null) {
						$SID2 = $userObjectId.onPremisesSecurityIdentifier;
					}		
					$SID = "{0}/{1}" -f $SID1, $SID2;
					Write-Host "nextLinkCount: $nextLinkCount userCount: $userCount ObjectId: $($userObjectId.id) ==> SIDs: $SID";
					$userIdentifiers.Add($SID) | Out-Null;
					$userCount += 1;
				}

				$nextLink = $null;
				$ErrorActionPreference = "Stop";
				Try {
					$nextLink = $graphResponse2.'@odata.nextLink';
					Write-Host "nextLink: $nextLink";
				}
				Catch {
					Write-Host "error (or end of stream)[2] getting odata.nextLink: $_";
				}
				Finally {
					$ErrorActionPreference = "Continue"
				}
			}
			$userCount -= 1;

			$body = $userIdentifiers -join ";";
			Write-Host "body: $body";
			Write-Host "userCount: $userCount";
			Write-Host "nextLinkCount: $nextLinkCount";
		}
	}
	else {
		Write-Host "groupId is null";
	}
}
else {
	Write-Host "AzGroupName is null";
}

Write-Host "Returning body: $body";

# Associate values to output bindings by calling 'Push-OutputBinding'.
Push-OutputBinding -Name Response -Value ([HttpResponseContext]@{
    StatusCode = [HttpStatusCode]::OK
    Body = $body
})
